
precision lowp int; 
precision lowp float; 


//Passed through from the VS
varying lowp vec4 uvs;
varying lowp vec3 worldnormal;
varying lowp vec4 vertexcolour;
varying lowp vec3 viewdirection;


//Various uniforms ahoy
uniform sampler2D IN_texture_diffuse;

#if defined(USE_TOON_SHADING)
uniform sampler2D IN_texture_toon;
uniform vec3 IN_toon_dir;
#endif

    
#if defined(USE_SPECULAR)
uniform sampler2D IN_texture_skymap_specular;

#if defined(USE_SPECULAR_SKIPPING)
uniform mediump vec3 IN_material_model_origin;
#endif 

uniform mediump float IN_material_specpow;
uniform lowp float IN_material_speclevel;
uniform lowp vec3 IN_material_speccolour;
uniform lowp vec3 IN_material_specdir;
#endif



uniform mediump vec3 IN_scene_eye_pos;
uniform lowp vec3 IN_scene_eye_dir;
uniform lowp float IN_scene_ambientscale;



#if defined(USE_RIM_SHADING)

uniform lowp float IN_material_rimscale;
uniform mediump float IN_material_rimpower;
uniform lowp vec3 IN_material_rimcolour;
#endif



#if defined(USE_SPECULAR)
lowp vec2 DoImageBasedSpecular(lowp vec3 reflect)
{   
	return vec2(  ((atan(reflect.z,reflect.x) * 0.1591549430 + 0.5))  ,  (-reflect.y * 0.5) +0.5 );
}
#endif 


void main() 
{

	lowp vec3 normalizedNormal = normalize(worldnormal);  //This gets optimized out if noones using it
    lowp vec3 viewDir = normalize(viewdirection);
   
#if defined(USE_DIFFUSE_TEXTURE)
   lowp vec4 diffuseSample = texture2D(IN_texture_diffuse, uvs.xy);
   lowp vec4 diffuse = diffuseSample * vertexcolour;
#else
   lowp vec4 diffuse = vertexcolour;
#endif



#if defined(USE_ALPHA_TEST)
	if (diffuse.a < 0.5)
	{
		discard;
	}
#endif	

#if defined(USE_TOON_SHADING)
	lowp float dotresult = (dot(normalizedNormal, IN_toon_dir) * 0.49) + 0.5;
	lowp vec4 toonSample = texture2D(IN_texture_toon, vec2(dotresult, 0.5));
	diffuse.xyz *= toonSample.xyz;
#endif



#if defined(USE_SPECULAR_SKIPPING) //image based with weird inputs to make it facety
    
	lowp vec3 reflectVector = -normalize( reflect(-IN_material_model_origin, normalizedNormal ));
    lowp vec2 specularLightUV = DoImageBasedSpecular(reflectVector);
    lowp vec3 specularAccumulation = texture2D(IN_texture_skymap_specular, specularLightUV).xyz  * IN_material_speclevel * IN_material_speccolour;

#elif defined(USE_SPECULAR_MATH)  //math based

    lowp vec3 reflectVector = reflect(normalizedNormal, viewDir);

	lowp float reflectValue = clamp( dot(reflectVector, IN_material_specdir), 0.001, 1.0);   // pow(0) == unhappy glsl on ipad 1
    lowp float specularValue = pow(reflectValue, IN_material_specpow) * IN_material_speclevel;
    lowp vec3 specularAccumulation =  specularValue * IN_material_speccolour;

#elif defined(USE_SPECULAR) //image based

    lowp vec3 reflectVector = reflect( normalizedNormal, viewDir);
    lowp vec2 specularLightUV = DoImageBasedSpecular(reflectVector);
    lowp vec3 specularAccumulation = texture2D(IN_texture_skymap_specular, specularLightUV).xyz * IN_material_speclevel * IN_material_speccolour;
	
#endif

#if defined(USE_RIM_SHADING)
    lowp vec3 rim = pow( 1.001 - abs( dot(viewDir, normalizedNormal) ), IN_material_rimpower ) * IN_material_rimscale * IN_material_rimcolour;  // pow(0) == unhappy glsl on ipad 1
    diffuse.xyz += rim;
#endif



    //Gather
#if defined(USE_SPECULAR) 
    gl_FragColor.xyz = (diffuse.xyz) + specularAccumulation;
#else
    gl_FragColor.xyz = diffuse.xyz;
#endif 
    
    
    
    gl_FragColor.w = diffuse.w;
    
}
